/*********************************************************************
 *                    Flowcode Accelerometer Component Code
 *
 * File: Custom_Code.c
 *
 * (c) 2007 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 271008 | BR | Created
 *        |    |
 *        |    |
 *        |    |
 *        |    |
 *
 *
 *
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   char
   short
   char*

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

 Macro substitutions:
   %a = ??
   %b = ??
   %c = ??
   %d = ??
   %e = ??
   %f = ??
   %g = ??
   %h = ??
   %i = ??
   %j = ??

**********************************************************************

[Settings]
CLSID={4B174520-75AB-4B14-A0F7-2AA6C203F1DC}
IsAnalogue=0
MultipleAllowed=1
Description=Flowcode Accelerometer Component

**********************************************************************

[Port]
DesiredPort=0
PortMandatory=0
SamePortAsID=0

[Pins]
Count=0

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4

[PinDesiredBit]

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3

[PinMustUsePort]
# boolean

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number

[PinMustUsePin]
# boolean

[PinValue]
# boolean (0 = off, 1 = on)

**********************************************************************

[MacroNames]
Count=8
1=GetDefines
2=Init_Bus
3=Update_XYZ_Data
4=Collect_XYZ_Data
5=Write_Register
6=Write_N_Registers
7=Read_Register
8=Read_N_Registers

[MacroReturns]
1=void
2=char
3=char
4=short
5=char
6=void
7=char
8=void

[MacroIsPrivate]
1=1
2=0
3=0
4=0
5=0
6=0
7=0
8=0

[MacroParameters_GetDefines]
Count=0

[MacroParamTypes_GetDefines]

[MacroParameters_Init_Bus]
Count=0

[MacroParamTypes_Init_Bus]

[MacroParameters_Update_XYZ_Data]
Count=0

[MacroParamTypes_Update_XYZ_Data]

[MacroParameters_Collect_XYZ_Data]
Count=1
1=axis

[MacroParamTypes_Collect_XYZ_Data]
1=char

[MacroParameters_Write_Register]
Count=2
1=reg
2=data

[MacroParamTypes_Write_Register]
1=char
2=char

[MacroParameters_Write_N_Registers]
Count=3
1=start_reg
2=num_regs
3=data

[MacroParamTypes_Write_N_Registers]
1=char
2=char
3=char*

[MacroParameters_Read_Register]
Count=1
1=reg

[MacroParamTypes_Read_Register]
1=char

[MacroParameters_Read_N_Registers]
Count=3
1=start_reg
2=num_regs
3=ret_data

[MacroParamTypes_Read_N_Registers]
1=char
2=char
3=char*

 ********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*InitialisationCode_Start*/
/*InitialisationCode_End*/

/*InterruptCode_Start*/
/*InterruptCode_End*/




/********************************************************************
 * FUNCTIONS
 ********************************************************************/


void GetDefines()
{
/*Macro_GetDefines_Start*/

}  //Dummy end of function to allow defines to be added correctly



//***************** ACCELEROMETER USER CONFIGURATION BEGIN *****************


//ACCELEROMETER COMPONENT DEFINES
#define MX_ACCEL_BUS	1//%a			//0=SPI, 1=I2C
#define MX_DATA_RATE	2//%b			//0=40hz, 1=160hz, 2=640hz, 3=2560hz
#define MX_SCALE		1//%c			//0=+/-2G, 1=+/-6G
#define MX_ADDRESS		0x3A//%d		//Default address 0x3A

#define MX_INT_PIN		2//%e
#define MX_INT_TRIS		trisb//
#define MX_INT_PORT		portb//%f

#ifdef AMX_ACCEL_BUS == 0				//SPI COMPONENT DEFINES

	#define MX_EN_PIN 		1//%g
	#define MX_EN_TRIS		trisc//
	#define MX_EN_PORT		portc//%h

#else									//I2C COMPONENT DEFINES

	#define MX_HARDWARE  	0//%i		//0=Sw, 1=Hw
	#define MX_MI2C_BAUD 	60//%j		//400khz

	#ifdef MX_HARDWARE == 0

		#define MX_PORT		portb//%k
		#define MX_TRIS 	trisb//%l
		#define MX_SCL  	0//%m
		#define MX_SDA  	1//%n

	#endif

#endif


//***************** ACCELEROMETER USER CONFIGURATION END *****************





char init_transaction(char mode, char reg, char multiple);
void end_transaction();


#ifdef AMX_ACCEL_BUS == 0

	#define MX_SPI_SSPM 	0
	#define MX_SPI_CKP 		16
	#define MX_SPI_CKE 		64
	#define MX_SPI_SMP 		0

	//SPI: //Macro function declarations
	void MX_SPI_Init();
	void MX_SPI_Send_Char(char Char);
	char MX_SPI_Get_Char();

	//internal function implementations
	void MX_SPI_Init()
	{

			//GetSPI_InitCode
			#ifndef MX_SPI
				#pragma error "Chip does not have SPI capability"
			#endif

			//initialise spi
			#ifdef MX_SPI_B
				set_bit(trisb,MX_SPI_SDI);
				clear_bit(trisb,MX_SPI_SDO);
				clear_bit(trisb,MX_SPI_SCK);
			#endif

			#ifdef MX_SPI_C
				set_bit(trisc,MX_SPI_SDI);
				clear_bit(trisc,MX_SPI_SDO);
				clear_bit(trisc,MX_SPI_SCK);
			#endif

			#ifdef MX_SPI_BCB
				set_bit(trisb,MX_SPI_SDI);
				clear_bit(trisc,MX_SPI_SDO);
				clear_bit(trisb,MX_SPI_SCK);
			#endif

			clear_bit(MX_EN_TRIS,MX_EN_PIN);

			//disable accelerometer
			set_bit(MX_EN_PORT, MX_EN_PIN);

			sspstat = SPI_CKE | SPI_SMP;

		  #ifdef SSPCON
			sspcon = SPI_SSPM | SPI_CKP;
			set_bit(sspcon, SSPEN);
		  #endif
		  #ifdef SSPCON1
			sspcon1 = SPI_SSPM | SPI_CKP;
			set_bit(sspcon1, SSPEN);
		  #endif

	}

	void MX_SPI_Send_Char(char Char)
	{
			//GetSPI_Send_CharCode
			#ifndef MX_SPI
				#pragma error "Chip does not have SPI capability"
			#endif

			//send the Char
			sspbuf = Char;

			//wait until sspbuf is empty
			while ((sspstat & 0x01) == 0);
	}

	char MX_SPI_Get_Char()
	{
			//GetSPI_Get_CharCode
			char retVal = 0;
			#ifndef MX_SPI
				#pragma error "Chip does not have SPI capability"
			#endif

			//send a dummy Char
			sspbuf = 0x5A;

			//wait until sspbuf is empty
			while ((sspstat & 0x01) == 0);

			//receive the Char
			retVal = sspbuf;

			return (retVal);
	}



#else

	//I2C COMPONENT DEFINES
	#define MX_MI2C_SMBUS  	0
	#define MX_MI2C_SLEW  	0

	#define MX_I2C_DELAY 	delay_10us(2);
	#define SET_SCL			set_bit(MX_TRIS,MX_SCL);
	#define CLEAR_SCL		{clear_bit(MX_TRIS,MX_SCL); clear_bit(MX_PORT,MX_SCL);}
	#define SET_SDA			set_bit(MX_TRIS,MX_SDA);
	#define CLEAR_SDA		{clear_bit(MX_TRIS,MX_SDA); clear_bit(MX_PORT,MX_SDA);}

	//I2C: //Macro function declarations
	void MX_MI2C_Init();
	void MX_MI2C_Start();
	void MX_MI2C_Restart();
	void MX_MI2C_Stop();
	char MX_MI2C_Transmit_Byte(char Data);
	char MX_MI2C_Receive_Byte(char Last);


	void MX_MI2C_Init()
	{
			#if MX_HARDWARE == 1									//Use Master I2C Hardware
				#ifndef MX_MI2C
					#warning "This chip does not have Master I2C capability"
				#else
					#ifdef MX_I2C_A
						#define MX_I2C_TRIS trisa
					#endif
					#ifdef MX_I2C_B
						#define MX_I2C_TRIS trisb
					#endif
					#ifdef MX_I2C_C
						#define MX_I2C_TRIS trisc
					#endif
					#ifdef MX_I2C_D
						#define MX_I2C_TRIS trisd
					#endif
					#if MX_MI2C_SLEW == 1
						clear_bit (sspstat,SMP);				//Slew Rate Control Enabled
					#else
						set_bit (sspstat,SMP);					//Slew Rate Control Disabled
					#endif
					#if MX_MI2C_SMBUS == 1
						set_bit (sspstat,CKE);					//Enable SMBus specific inputs
					#else
						clear_bit (sspstat,CKE);				//Disable SMBus specific inputs
					#endif

				  #ifdef SSPCON
					sspcon = 0x28;								//Setup I2C into Master Mode
				  #endif
				  #ifdef SSPCON1
					sspcon1 = 0x28;								//Setup I2C into Master Mode
				  #endif

					sspadd = MX_MI2C_BAUD;						//Set the Baud Rate
					sspcon2 = 0x00;								//Clear the control bits
					set_bit(intcon,7);
					set_bit(MX_I2C_TRIS, MX_I2C_SCL);			//Configure SCL as Input
					set_bit(MX_I2C_TRIS, MX_I2C_SDA);			//Configure SDA as Input
				#endif
			#else												//Use Software
		    	clear_bit(MX_PORT,MX_SCL);						//Set Outputs to 0
				clear_bit(MX_PORT,MX_SDA);
				SET_SDA;										//Configure SDA as Input
				SET_SCL;										//Configure SCL as Input
			#endif
	}

	void MX_MI2C_Start()
	{
			#if MX_HARDWARE == 1									//Use Master I2C Hardware
				#ifndef MX_MI2C
					#warning "This chip does not have Master I2C capability"
				#else
					clear_bit(pir1,SSPIF);						//Clear SSP interrupt flag
					set_bit(sspcon2,SEN);						//Initiate start condition
					while(test_bit(sspcon2,SEN));				//Wait for start bit to be generated
				#endif
			#else												//Use Software I2C
				SET_SCL;										//Make Sure SCL is Set
				SET_SDA;										//Make Sure SDA is Set
				MX_I2C_DELAY;									//Small Delay
				CLEAR_SDA;										//Clear SDA
				MX_I2C_DELAY;
				CLEAR_SCL;										//Clear SCL
				MX_I2C_DELAY;
			#endif
	}

	void MX_MI2C_Restart()
	{
			#if MX_HARDWARE == 1									//Use Master I2C Hardware
				#ifndef MX_MI2C
					#warning "This chip does not have Master I2C capability"
				#else
					clear_bit(pir1,SSPIF);						//Clear SSP interrupt flag
					set_bit(sspcon2,RSEN);						//Initiate restart condition
					while(test_bit(sspcon2,RSEN));				//Wait for restart bit to be generated
				#endif
			#else												//Use Software I2C
				SET_SDA;										//Set SDA High
				MX_I2C_DELAY;
				SET_SCL;										//Set SCL High
				MX_I2C_DELAY;
				CLEAR_SDA;										//Set SDA Low
				MX_I2C_DELAY;
				CLEAR_SCL;										//Set SCL Low
				MX_I2C_DELAY;
			#endif
	}

	void MX_MI2C_Stop()
	{
			#if MX_HARDWARE == 1									//Use Master I2C Hardware
				#ifndef MX_MI2C
					#warning "This chip does not have Master I2C capability"
				#else
					clear_bit(pir1,SSPIF);						//Clear SSP interrupt flag
					set_bit(sspcon2,PEN);						//Initiate stop condition
					while(test_bit(sspcon2,PEN));				//Wait for stop bit to be generated
					delay_ms(10);								//Wait before reusing the I2C BUS
				#endif
			#else												//Use Software I2C
				CLEAR_SCL;										//Set SCL Low
				CLEAR_SDA;										//Set SDA Low
				MX_I2C_DELAY;
				SET_SCL;										//Set SCL High
				MX_I2C_DELAY;
				SET_SDA;										//Set SDA High
				delay_ms(10);									//Wait before reusing the I2C BUS
			#endif
	}

	char MX_MI2C_Transmit_Byte(char Data)
	{
			#if MX_HARDWARE == 1									//Use Master I2C Hardware
				#ifndef MX_MI2C
					#warning "This chip does not have Master I2C capability"
				#else
					clear_bit(pir1,SSPIF);						//Clear SSP interrupt flag
					sspbuf=Data;								//Send byte
					while(test_bit(pir1,SSPIF) == 0);			//Wait for control bit to be sent
					if(test_bit(sspcon2,ACKSTAT))				//Check Acknowledgement
						return (1);								//No Acknowledgement
					else return (0);							//Acknowledgement received
				#endif
			#else												//Use Software I2C
				char bit_mask;
				char local_ack = 0;
				for(bit_mask=0x80;bit_mask;bit_mask=bit_mask>>1)
				{
					if(Data & bit_mask)
						SET_SDA;								//Set SDA High
					else CLEAR_SDA;								//Set SDA Low
					MX_I2C_DELAY;
					SET_SCL;									//Set SCL High
					MX_I2C_DELAY;
					CLEAR_SCL;									//Set SCL Low
					MX_I2C_DELAY;
				}
				SET_SDA;										//Set SDA High
				MX_I2C_DELAY;
				SET_SCL;										//Set SCL High
				MX_I2C_DELAY;
				if (test_bit(MX_PORT,MX_SDA))					//Get the Status Bit
					local_ack = 1;
				MX_I2C_DELAY;
				CLEAR_SCL;										//Set SCL Low
				MX_I2C_DELAY;
				return (local_ack);								//0 = Ack received : 1 = Ack not received
			#endif
	}

	char MX_MI2C_Receive_Byte(char Last)
	{
			#if MX_HARDWARE == 1									//Use Master I2C Hardware
				#ifndef MX_MI2C
					#warning "This chip does not have Master I2C capability"
				#else
					clear_bit(pir1,SSPIF);						//Clear SSP interrupt flag
					set_bit(sspcon2,RCEN);						//Initiate Read
					while(test_bit(pir1,SSPIF) == 0);			//Wait for data read
					if (Last)
						set_bit(sspcon2,ACKDT);					//Send Nack
					else clear_bit(sspcon2,ACKDT);				//Send Ack
					set_bit(sspcon2,ACKEN);						//Initiate Nack
					while(test_bit(sspcon2,ACKEN));				//Wait for data read
					return(sspbuf);								//Store incoming data
				#endif
			#else												//Use Software I2C
				char bit_mask;
				char Data = 0;
				char local_ack;
				SET_SDA;										//Set SDA High
				MX_I2C_DELAY;
				for(bit_mask=0x80;bit_mask;bit_mask=bit_mask>>1)
				{
					SET_SCL;									//Set SCL High
					MX_I2C_DELAY;
					SET_SDA;
					if(test_bit(MX_PORT,MX_SDA))				//Read Incoming Bit
						Data = Data | bit_mask;					//Add data to variable
					CLEAR_SCL;									//Set SCL Low
					MX_I2C_DELAY;
				}
				if (Last)										//Acknowledge?
					SET_SDA;									//Send Nack
				else CLEAR_SDA;									//Send Ack
				MX_I2C_DELAY;
				SET_SCL;										//Set SCL High
				MX_I2C_DELAY;
				CLEAR_SCL;										//Set SCL Low
				MX_I2C_DELAY;
				return (Data);
			#endif
	}


#endif


char init_transaction(char mode, char reg, char multiple)
{
	char temp_addr;
	char ack = 1;

	#ifdef AMX_ACCEL_BUS == 0				//SPI TRANSACTION

		clear_bit(MX_EN_PORT, MX_EN_PIN);

		if(multiple)
			reg = reg | 0x40;
		if (mode)
			reg = reg | 0x01;

		MX_SPI_Send_Char(reg);
		return 0;

	#else									//I2C TRANSACTION

		MX_MI2C_Start();					//Start I2C
		MX_MI2C_Transmit_Byte(MX_ADDRESS);	//Send address and read / write command

		if(multiple)
			reg = reg | 0x80;

		ack = MX_MI2C_Transmit_Byte(reg);			//Send register and single / multiple command
		if (ack)
			return 255;

		if (mode)
		{
			temp_addr = MX_ADDRESS | 0x01;	//Calculate address and read / write command
			MX_MI2C_Restart();
			ack = MX_MI2C_Transmit_Byte(temp_addr);
		}
		return ack;
	#endif
}


void end_transaction()
{
	#ifdef AMX_ACCEL_BUS == 0				//SPI TRANSACTION
		set_bit(MX_EN_PORT, MX_EN_PIN);
	#else									//I2C TRANSACTION
		MX_MI2C_Stop();
	#endif
}




//global variables
short MX_X_Var = 0;
short MX_Y_Var = 0;
short MX_Z_Var = 0;



// Dummy function to close the defines section off
void CUSTOM_Dummy_Function();
void CUSTOM_Dummy_Function()
{

/*Macro_GetDefines_End*/
}



char Init_Bus(void)
{
/*Macro_Init_Bus_Start*/

	char temp_reg = 0xC7;
	char RetVal;

	set_bit(MX_INT_TRIS, MX_INT_PIN);

	#ifdef AMX_ACCEL_BUS == 0
		MX_SPI_Init();
	#else
		MX_MI2C_Init();
	#endif

	temp_reg = temp_reg | (MX_DATA_RATE << 4);
	RetVal = Write_Register(0x20, temp_reg);	//startup device

	if (RetVal)
		return 255;

	temp_reg = MX_SCALE << 7;
	temp_reg = temp_reg | 0x44;
	RetVal = Write_Register(0x21, temp_reg);	//scale selection

	if (RetVal)
		return 254;

	return 0;

/*Macro_Init_Bus_End*/
}


char Update_XYZ_Data(void)
{
/*Macro_Update_XYZ_Data_Start*/
	char data_array[6];

	if(test_bit(MX_INT_PORT, MX_INT_PIN))
	{
		MX_X_Var = FCD_Custom0_Read_Register(0x28);
		MX_X_Var = MX_X_Var | ( FCD_Custom0_Read_Register(0x29) << 8);
		MX_Y_Var = FCD_Custom0_Read_Register(0x2A);
		MX_Y_Var = MX_Y_Var | ( FCD_Custom0_Read_Register(0x2B) << 8);
		MX_Z_Var = FCD_Custom0_Read_Register(0x2C);
		MX_Z_Var = MX_Z_Var | ( FCD_Custom0_Read_Register(0x2D) << 8);

		return 1;
	}
	else
		return 0;
/*Macro_Update_XYZ_Data_End*/
}


short Collect_XYZ_Data(char axis)
{
/*Macro_Collect_XYZ_Data_Start*/
	switch(axis)
	{
		case 0:
		case 'x':
		case 'X':
			return MX_X_Var;
		case 1:
		case 'y':
		case 'Y':
			return MX_Y_Var;
		case 2:
		case 'z':
		case 'Z':
			return MX_Z_Var;
	}
	return -1;
/*Macro_Collect_XYZ_Data_End*/
}


void Write_Register(char reg, char data)
{
/*Macro_Write_Register_Start*/

	char RetVal;

	RetVal = init_transaction(0, reg, 0);

	if (RetVal)
		return 255;

	#ifdef AMX_ACCEL_BUS == 0				//SPI TRANSACTION
		MX_SPI_Send_Char(data);
	#else									//I2C TRANSACTION
		RetVal = MX_MI2C_Transmit_Byte(data);
	#endif
	end_transaction();

	return RetVal;

/*Macro_Write_Register_End*/
}


void Write_N_Registers(char start_reg, char num_regs, char* data)
{
/*Macro_Write_N_Registers_Start*/
	char i;
	char j = 0;
	init_transaction(0, start_reg, 1);
	for(i=start_reg; i<(start_reg + num_regs); i++)
	{
		#ifdef AMX_ACCEL_BUS == 0				//SPI TRANSACTION
			MX_SPI_Send_Char(data[j]);
		#else									//I2C TRANSACTION
			MX_MI2C_Transmit_Byte(data[j]);
		#endif
		j = j + 1;
	}
	end_transaction();
/*Macro_Write_N_Registers_End*/
}


char Read_Register(char reg)
{
/*Macro_Read_Register_Start*/
	char Retval;
	init_transaction(1, reg, 0);

	#ifdef AMX_ACCEL_BUS == 0				//SPI TRANSACTION
		Retval = MX_SPI_Get_Char();
	#else									//I2C TRANSACTION
		Retval = MX_MI2C_Receive_Byte(1);
	#endif

	end_transaction();
	return Retval;
/*Macro_Read_Register_End*/
}


void Read_N_Registers(char start_reg, char num_regs, char* ret_data)
{
/*Macro_Read_N_Registers_Start*/
	char i;
	char j = 0;
	char last = 0;
	init_transaction(1, start_reg, 1);
	for(i=start_reg; i<(start_reg+num_regs); i++)
	{
		#ifdef AMX_ACCEL_BUS == 0				//SPI TRANSACTION
			ret_data[j] = MX_SPI_Get_Char();
		#else									//I2C TRANSACTION
			if(j == (num_regs - 1))
				last = 1;
			ret_data[j] = MX_MI2C_Receive_Byte(last);
		#endif
		j = j + 1;
	}
	end_transaction();
/*Macro_Read_N_Registers_End*/
}



















